﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Net;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Documents;
using System.Windows.Input;
using System.Windows.Media;
using System.Windows.Media.Animation;
using System.Windows.Shapes;
using System.Windows.Threading;

namespace ColorRectangles
{
    public partial class MainPage : UserControl
    {
        const int BOARD_WIDTH = 10;
        const int BOARD_HEIGHT = 10;

        SilverlightControlRectangle[,] board = new SilverlightControlRectangle[BOARD_WIDTH, BOARD_HEIGHT];
        Random rnd = new Random();
        DispatcherTimer timer = new DispatcherTimer();

        int time;
        int score;
        bool gameOver;

        private void CreateGrid()
        {
            GridGame.Background = GraphicsHelper.CreateLinearGradientBrush(Color.FromArgb(255, 180, 180, 180), Color.FromArgb(255, 20, 20, 20));

            for (int i = 0; i < BOARD_WIDTH; i++)
            {
                ColumnDefinition c = new ColumnDefinition();
                GridGame.ColumnDefinitions.Add(c);
            }

            for (int i = 0; i < BOARD_HEIGHT; i++)
            {
                RowDefinition r = new RowDefinition();
                GridGame.RowDefinitions.Add(r);
            }
        }

        private SilverlightControlRectangle NewRectangle(int column, int row)
        {
            SilverlightControlRectangle r = new SilverlightControlRectangle();

            r.SetValue(Grid.ColumnProperty, column);
            r.SetValue(Grid.RowProperty, row);
            r.RandomColor();
            r.MouseLeftButtonDown += r_MouseLeftButtonDown;

            return r;
        }

        private void CreateRectangles()
        {
            GridGame.Children.Clear();

            for (int i = 0; i < BOARD_WIDTH; i++)
            {
                for (int j = 0; j < BOARD_HEIGHT; j++)
                {
                    SilverlightControlRectangle r = NewRectangle(i, j);

                    board[i, j] = r;

                    GridGame.Children.Add(r);
                }
            }
        }

        private void FillRectangles()
        {
            for (int i = 0; i < BOARD_WIDTH; i++)
            {
                for (int j = 0; j < BOARD_HEIGHT; j++)
                {
                    if (board[i, j] == null)
                    {
                        SilverlightControlRectangle r = NewRectangle(i, j);

                        board[i, j] = r;

                        GridGame.Children.Add(r);
                    }
                }
            }
        }

        void RandomColors()
        {
            for (int i = 0; i < BOARD_WIDTH; i++)
            {
                for (int j = 0; j < BOARD_HEIGHT; j++)
                {
                    if (board[i, j] != null && board[i, j].Visibility == Visibility.Visible)
                    {
                        if (rnd.Next(100) < 40)
                        {
                            board[i, j].RandomColor();
                        }
                    }
                }
            }
        }

        void ScrollColumn(int column, int row)
        {
            for (int tmprow = row - 1; tmprow >= 0; tmprow--)
            {
                if (board[column, tmprow] != null)
                {
                    if (board[column, tmprow + 1] == null || board[column, tmprow + 1].Visibility == Visibility.Collapsed)
                    {
                        board[column, tmprow].SetValue(Grid.RowProperty, tmprow + 1);
                        board[column, tmprow + 1] = board[column, tmprow];
                        board[column, tmprow] = null;
                    }
                }
            }
        }

        private void ShowData()
        {
            TextBlockScore.Text = score.ToString();
            TextBlockTime.Text = time.ToString();
        }

        private void ResetGame()
        {
            timer.Stop();

            score = 0;
            CreateRectangles();

            time = 10;
            ShowData();

            gameOver = false;

            timer.Start();
        }

        void r_MouseLeftButtonDown(object sender, MouseButtonEventArgs e)
        {
            SilverlightControlRectangle r = sender as SilverlightControlRectangle;

            if (r.RectangleColor == Colors.Gray || gameOver)
            {
                SoundGray.Position = TimeSpan.Zero;
                SoundGray.Play();

                return;
            }

            r.Visibility = Visibility.Collapsed;

            int column = (int)r.GetValue(Grid.ColumnProperty);
            int row = (int)r.GetValue(Grid.RowProperty);

            board[column, row] = null;

            ScrollColumn(column, row);

            score += 100;

            if (r.RectangleColor == Colors.Red)
            {
                RandomColors();

                SoundRed.Position = TimeSpan.Zero;
                SoundRed.Play();
            }
            else if (r.RectangleColor == Colors.Green)
            {
                if (time < 10)
                {
                    time += 1;
                }

                SoundGreen.Position = TimeSpan.Zero;
                SoundGreen.Play();
            }
            else if (r.RectangleColor == Colors.Blue)
            {
                score += 100;

                SoundBlue.Position = TimeSpan.Zero;
                SoundBlue.Play();
            }
            else if (r.RectangleColor == Colors.Orange)
            {
                FillRectangles();

                SoundOrange.Position = TimeSpan.Zero;
                SoundOrange.Play();
            }

            ShowData();
        }

        private void StackPanel_MouseLeftButtonDown(object sender, MouseButtonEventArgs e)
        {
            ResetGame();

            SoundNewGame.Position = TimeSpan.Zero;
            SoundNewGame.Play();
        }

        void timer_Tick(object sender, EventArgs e)
        {
            if (time > 1)
            {
                time--;

                ShowData();
            }
            else
            {
                timer.Stop();

                gameOver = true;

                ChildWindowGameOver DialogGameOver = new ChildWindowGameOver(score);
                DialogGameOver.Show();
            }
        }

        public MainPage()
        {
            InitializeComponent();

            GridData.Background = GraphicsHelper.CreateLinearGradientBrush(Color.FromArgb(255, 50, 50, 50), Color.FromArgb(255, 180, 180, 180));

            CreateGrid();

            timer.Interval = TimeSpan.FromSeconds(1);
            timer.Tick += timer_Tick;

            ResetGame();
        }
    }
}
